/*
  generic outputdevice for yaf
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */



#ifndef __OUTPUTDEVICE_H
#define __OUTPUTDEVICE_H

#include <devices/nodeDevice.h>
#include <filter/filter.h>

#include <qobject.h>
#include <qsocknot.h> 
#include <pthread.h>


/**
  What are delivermodes?
  
  <p>
  With the deliverModes you can control the behaviour of
  an outputDevice
  If the deliverMode is _WITH_NOTIFY the device sends notify
  messages.
  If the deliverMode is _SINGLE_ and the device is 
  a streamProducer  the streamProducer stops after
  each delivery and you must wakeUp the streamProducer.
  _SINGLE_ has no effect on a normal device.
  <p>
  These delivery methods give you more controll about
  a device.
  Eg. A SpectrumAnalyser is very time consuming.
  If the user thinks he should save cpu cycles instead
  of watching the SpectrumAnalyser he sets _NO_NOTIFY
  And the SpectrumAnalyer stops analysing, without
  removing the device from the device graph.
  <p>
  For a streamProducer the _SINGLE_ delivery is usefull
  if you must force synchronous delivery because the gui
  may be sometimes to slow to get all notiefies by the
  devices.
  Eg. A device which should exactly show the pcm data
  in a time diagramm.The user must be sure that the data
  he sees is the real stream and that not a single pcm
  chunk is missing because the gui was to slow to 
  catch the display information. With the single delivery
  you can first update the gui and then you can allow
  the producer to write the next pcm chunk through the
  device graph. Note: SINGLE delivery usally creates skips
  in the music.
*/
  


#define _DELIVER_MODE_NO_NOTIFY    1
#define _DELIVER_MODE_WITH_NOTIFY  2
#define _DELIVER_MODE_SINGLE       3




class OutputDevice : public NodeDevice {
 Q_OBJECT 

  int deliverMode;
  int fd[2];
  int lData;
  QSocketNotifier *sn;
  
  pthread_mutex_t dataMut;


 public:
  OutputDevice();
  virtual ~OutputDevice();

  char* getNodeName();

  virtual int openDevice();
  virtual int closeDevice();
  virtual void jump(float time);

  void setDeliverMode(int deliverMode);
  int getDeliverMode();
 
  int getData();

  /**
     usefull for the algorithms in the device
  */
  Accelerator* getAccelerator();

 signals:
  void dataNotify();

 public slots:
  /** 
       call this to inform the gui about data
  */
  void dataSendNotify();  
  void dataReceivedNotify();
  void clearData();
};

#endif
  
  



