//
//  kmpg
//
//  Copyright (C) 1997 Bjarni Ivarsson
//  email: Bjarni.Ivarsson@swipnet.se
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef KMPG_H
#define KMPG_H

// Internal include files
//#include "playlistmgr.h"
#include "equalizer.h"
#include "playlist.h"
#include "prefs.h"
#include "ledpanel.h"
#include <frontendDecoderSPlay.h>

// QT include files
#include <qfiledlg.h>
#include <qpopmenu.h>
#include <qaccel.h>
#include <qpixmap.h>

// KDE include files
#include <kapp.h>
#include <drag.h>
#include <kurl.h>
#include <kmenubar.h>
#include <ktopwidget.h>

class KMpg : public KTopLevelWidget
{
      Q_OBJECT
   public:
      KMpg( QWidget *parent = 0, const char *name = 0 );
      ~KMpg();
   private:
      virtual void saveProperties( KConfig* );
      virtual void readProperties( KConfig* );
      
   protected:
      void setupMenubar();
      void closeEvent ( QCloseEvent *);

   public slots:
      void showMenubar();
      void showTrackCal();
      void showSlider();
      void showControls();
      
      void openUrl( KURL, bool );

      // Callbacks for the decoder...
      void stopped(FrontendDecoder::stopReason); // The decoder has stopped

      // Called when the ledpanel changes size
      void ledPanelResized();

      void openClicked();

      // Called when files are droppen on the main window
      void onDrop( KDNDDropZone* );
   private:
      // Items in main dialog
      LedPanel *ledPanel;
      KMenuBar *menuBar;
      QPopupMenu *file, *view;

      // DND zone
      KDNDDropZone *dropZone;

      // Other dialogs
      Playlist *playlistDlg;
      PlaylistMgr *playlistMgr;
      Preferences *prefsdialog;
      Equalizer *equalizer;
      QFileDialog *filedialog;
      FrontendDecoder *decoder;

      int showMenubar_id, showTrackCal_id, showSlider_id, showControls_id;
};

#endif
