//
//  kmpg
//
//  Copyright (C) 1997 Bjarni Ivarsson
//  email: Bjarni.Ivarsson@swipnet.se
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <iostream.h>

#include <kwm.h>

#include "prefs.h"

Preferences::Preferences( QWidget *parent ) :
   QTabDialog( parent )
{
   setCancelButton();
   setApplyButton();
   setOkButton();

   createPage1();
   createPage2();
   createPage3();
   addTab( page1, klocale->translate("General") );
   addTab( page2, klocale->translate("MPEG Decoder") );
   addTab( page3, klocale->translate("Audio Device") );

   connect( this, SIGNAL(applyButtonPressed()), this, SLOT(slotApply()));
   connect( this, SIGNAL(cancelButtonPressed()), this, SLOT(slotCancel()));

   initPrefs();
   slotChange();
   applyPrefs();

   // Disable session management for this window
   KWM::setWmCommand( handle(), "" );
}

Preferences::~Preferences()
{
//   cerr << "Preferences::~Preferences()" << endl;
}

void Preferences::createPage1()
{
   int tabwidth=320;
   
   page1 = new QWidget( this );
   QButtonGroup *grpbox1_1 = new QButtonGroup( "HTTP proxy", page1 );
   QLabel *lblServer = new QLabel( "Server", grpbox1_1 );
   lblServer->adjustSize();
   lblServer->move( 10, 20 );
   QLabel *lblPort = new QLabel( "Port", grpbox1_1 );
   lblPort->adjustSize();
   lblPort->move( tabwidth-65-lblServer->width() , 20 );   
   leHttpProxy = new QLineEdit( grpbox1_1 );
   leHttpProxy->move( 15+lblServer->width(), 18 );
   leHttpProxy->resize( lblPort->x()-leHttpProxy->x()-5, 
			leHttpProxy->height()-8 );
   leHttpProxyPort = new QLineEdit( grpbox1_1 );
   leHttpProxyPort->move( lblPort->x()+lblPort->width()+5, 18 );
   leHttpProxyPort->resize( tabwidth - 35 - leHttpProxyPort->x(), 
			    leHttpProxyPort->height()-8);
   
   grpbox1_1->setGeometry( 10, 10, tabwidth-20, lblServer->height()+30 );

   cbPlayOnOpen = new QCheckBox( klocale->translate("Play files on open"), page1 );
   cbPlayOnOpen->adjustSize();
   rbHeight = cbPlayOnOpen->height();
   cbPlayOnOpen->move( 10, grpbox1_1->height() + 25 );
   cbReadID3 = new QCheckBox( klocale->translate("Read ID3 songname tag"), page1 );
   cbReadID3->adjustSize();
   cbReadID3->move( 10, grpbox1_1->height() + 25 + rbHeight );
   cbAutoscroll = new QCheckBox( klocale->translate("Autoscroll songname"), page1 );
   cbAutoscroll->adjustSize();
   cbAutoscroll->move( 10, grpbox1_1->height() + 25 + 2*rbHeight );

   connect( leHttpProxy, SIGNAL(textChanged(const char*)), SLOT(slotChange(const char*)) );
   connect( leHttpProxyPort, SIGNAL(textChanged(const char*)), SLOT(slotChange(const char*)) );
   connect( cbPlayOnOpen, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( cbAutoscroll, SIGNAL(clicked()), SLOT(slotChange()) );

   page1->setFixedSize( tabwidth, 
			cbAutoscroll->y()+cbAutoscroll->height()+10 );
}

void Preferences::createPage2()
{
   int tabwidth=320;

   page2 = new QWidget( this );
   QButtonGroup *grpbox1 = new QButtonGroup( klocale->translate("Decode in"), page2 );
   rbStereo = new QRadioButton( klocale->translate("Stereo"), grpbox1 );
   rbStereo->move( 10, 20 );
   rbStereo->adjustSize();
   rbMono = new QRadioButton( klocale->translate("Mono"), grpbox1 );
   rbMono->adjustSize();
   rbMono->move( 10, 20 + rbHeight );
   rbMonoLeft = new QRadioButton( klocale->translate("Left"), grpbox1 );
   rbMonoLeft->adjustSize();
   rbMonoLeft->move( 25, 20 + rbHeight*2 );
   rbMonoRight = new QRadioButton( klocale->translate("Right"), grpbox1 );
   rbMonoRight->adjustSize();
   rbMonoRight->move( 25, 20 + rbHeight*3 );
   rbMonoMix = new QRadioButton( klocale->translate("Mix"), grpbox1 );
   rbMonoMix->adjustSize();
   rbMonoMix->move( 25, 20 + rbHeight*4 );
   grpbox1->setGeometry( 10, 10, tabwidth/2-15, 25+rbHeight*5 );
   buttonGroup = new QButtonGroup();
   buttonGroup->insert( rbMonoLeft, -2 );
   buttonGroup->insert( rbMonoRight, -2 );
   buttonGroup->insert( rbMonoMix, -2 );

   QButtonGroup *grpbox2 = new QButtonGroup( klocale->translate("Downsampling"), page2 );
   rbDS1 = new QRadioButton( klocale->translate("None"), grpbox2 );
   rbDS1->move( 10, 20 );
   rbDS1->adjustSize();
   rbDS2 = new QRadioButton( klocale->translate("1:2"), grpbox2 );
   rbDS2->move( 10, 20 + rbHeight );
   rbDS2->adjustSize();
   rbDS3 = new QRadioButton( klocale->translate("1:4"), grpbox2 );
   rbDS3->move( 10, 20 + 2*rbHeight );
   rbDS3->adjustSize();
   grpbox2->setGeometry( tabwidth/2+5, 10, tabwidth/2-15, grpbox1->height() );

   cbRangeViolation = new QCheckBox( klocale->translate("Check range violations"), page2 );
   cbRangeViolation->adjustSize();
   cbRangeViolation->move( 10, grpbox2->height() + 25 );
   cbResync = new QCheckBox( klocale->translate("Try to resync on error"), page2 );
   cbResync->adjustSize();
   cbResync->move( 10, grpbox2->height() + 25 + rbHeight );
   cbScalefactor = new QCheckBox( klocale->translate("Change scalefactor"), page2 );
   cbScalefactor->adjustSize();
   cbScalefactor->move( 10, grpbox2->height() + 25 + 2*rbHeight );
   leScalefactor = new QLineEdit( page2 );
   leScalefactor->adjustSize();
   leScalefactor->setGeometry( 25, grpbox2->height() + 25 + 3*rbHeight, 
			       tabwidth/2-45, leScalefactor->height() );
   connect( rbStereo, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( rbMono, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( rbMonoLeft, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( rbMonoRight, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( rbMonoMix, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( rbDS1, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( rbDS2, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( rbDS3, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( cbRangeViolation, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( cbResync, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( cbScalefactor, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( leScalefactor, SIGNAL(textChanged(const char*)), SLOT(slotChange(const char*)) );

   page2->setFixedSize( tabwidth, 
			leScalefactor->y()+leScalefactor->height()+10 );
}

void Preferences::createPage3()
{
   int tabwidth = 320, tabheight = 250;

   page3 = new QWidget( this );
   QButtonGroup *grpbox3_1 = new QButtonGroup( klocale->translate("Audio device"), page3 );
   rbDeviceDefault = new QRadioButton( klocale->translate("Default"), grpbox3_1 );
   rbDeviceDefault->move( 10, 20 );
   rbDeviceDefault->adjustSize();
   rbDeviceOther = new QRadioButton( klocale->translate("Other"), grpbox3_1 );
   rbDeviceOther->adjustSize();
   rbDeviceOther->move( 10, 20 + rbHeight );
   leDevice = new QLineEdit( grpbox3_1 );
   leDevice->adjustSize();
   leDevice->setGeometry( 25, 20 + 2*rbHeight, 
			  tabwidth/2-45, leDevice->height());
   grpbox3_1->setGeometry( 10, 10, 
			   tabwidth/2-15, 25+rbHeight*2+leDevice->height() );

   QButtonGroup *grpbox3_2 = new QButtonGroup( klocale->translate("Output to"), page3 );
   rbSpeaker = new QRadioButton( klocale->translate("Speaker"), grpbox3_2 );
   rbSpeaker->move( 10, 20 );
   rbSpeaker->adjustSize();
   rbLineout = new QRadioButton( klocale->translate("Line-out"), grpbox3_2 );
   rbLineout->move( 10, 20 + rbHeight );
   rbLineout->adjustSize();
   rbHeadphones = new QRadioButton( klocale->translate("Headphones"), grpbox3_2 );
   rbHeadphones->move( 10, 20 + 2*rbHeight );
   rbHeadphones->adjustSize();
   grpbox3_2->setGeometry( tabwidth/2+5, 10, 
			   tabwidth/2-15, grpbox3_1->height() );

   QButtonGroup *grpbox3 = new QButtonGroup( klocale->translate("Buffer"), page3 );
   grpbox3->setGeometry( 10, 20+grpbox3_1->height(), tabwidth-20, 60 );
   slider = new KSlider( KSlider::Horizontal, grpbox3 );
   slider->setGeometry( 10, 20, grpbox3->width()-20, 20 );
   slider->setRange( 0, 64 );
   slider->setSteps( 8, 64 );
   QLabel *lbl2_a = new QLabel( "0 Kb", grpbox3 );
   lbl2_a->adjustSize();
   lbl2_a->move( 10, 40 );
   QLabel *lbl2_b = new QLabel( "1024 Kb", grpbox3 );
   lbl2_b->adjustSize();
   lbl2_b->move( slider->width()-lbl2_b->width()+10, 40 );
   lbl2_b->setAlignment( AlignRight );

   connect( rbDeviceDefault, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( rbDeviceOther, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( leDevice, SIGNAL(textChanged(const char*)),
	    SLOT(slotChange(const char*)) );
   connect( rbSpeaker, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( rbLineout, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( rbHeadphones, SIGNAL(clicked()), SLOT(slotChange()) );
   connect( slider, SIGNAL(valueChanged(int)), SLOT(slotChange(int)) );
   
   page3->setFixedSize( tabwidth, 
			grpbox3->y()+grpbox3->height()+10 );
}

void Preferences::initPrefs()
{
   KConfig *config = kapp->getConfig();
   QString string;
   int integer;

   config->setGroup("General");

   _httpProxy = config->readEntry( "HTTPServer", "server.domain.com" );
   leHttpProxy->setText( _httpProxy );

   _httpPort = config->readNumEntry( "HTTPPort", 8080 );
   string.setNum( _httpPort );
   leHttpProxyPort->setText( string.copy() );

   _playOnOpen = config->readNumEntry( "PlayOnOpen", TRUE );
   cbPlayOnOpen->setChecked( _playOnOpen );

   _readID3 = config->readNumEntry( "ReadID3", TRUE );
   cbReadID3->setChecked( _readID3 );

   _autoscroll = config->readNumEntry( "Autoscroll", TRUE );
   cbAutoscroll->setChecked( _autoscroll );
      
   config->setGroup("Decoder");

   _forceToMono = config->readNumEntry( "ForceToMono", FALSE );
   rbMono->setChecked( _forceToMono );
   rbStereo->setChecked( !_forceToMono );

   _monoMode = config->readNumEntry( "MonoMode", 3 );
   rbMonoLeft->setChecked( _monoMode==1 );
   rbMonoRight->setChecked( _monoMode==2 );
   rbMonoMix->setChecked( _monoMode==3 );

   _downsample = config->readNumEntry( "Downsample", 1 );
   rbDS1->setChecked( _downsample==1 );
   rbDS2->setChecked( _downsample==2 );
   rbDS3->setChecked( _downsample==3 );

   _checkRangeViolations = config->readNumEntry( "CheckRangeViolations", 0 );
   cbRangeViolation->setChecked( _checkRangeViolations );

   _Resync = config->readNumEntry( "Resync", 0 );
   cbResync->setChecked( _Resync );

   _changeScalefactor = config->readNumEntry( "ChangeScalefactor", 0 );
   cbScalefactor->setChecked( _changeScalefactor );

   _scalefactor = config->readNumEntry( "Scalefactor", 32768 );
   string.setNum( _scalefactor );
   leScalefactor->setText( string.copy() );
   
   config->setGroup("AudioDevice");

   _useDefaultAudioDevice = config->readNumEntry( "AudioDeviceDefault", 1 );
   rbDeviceDefault->setChecked( _useDefaultAudioDevice );
   rbDeviceOther->setChecked( !_useDefaultAudioDevice );
   
   _audioDevice = config->readEntry( "AudioDevice", "dev/dsp" );
   leDevice->setText( _audioDevice );
      
   _outputTo = config->readNumEntry( "OutputTo", 1 );
   rbSpeaker->setChecked( _outputTo==1 );
   rbLineout->setChecked( _outputTo==2 );
   rbHeadphones->setChecked( _outputTo==3 );
   
   _bufferSize = config->readNumEntry( "Buffer", 128 );
   slider->setValue( _bufferSize/16 );

   prefsChanged = false;
}

void Preferences::applyPrefs()
{
   KConfig *config = KApplication::getKApplication()->getConfig();
   QString string;
   int integer;

   if (!prefsChanged)
      return;

   _httpProxy = leHttpProxy->text();
   _httpPort = QString(leHttpProxyPort->text()).toInt();
   _playOnOpen = cbPlayOnOpen->isChecked();
   _readID3 = cbReadID3->isChecked();
   _autoscroll = cbAutoscroll->isChecked();
   
   _forceToMono = rbMono->isChecked();
   _monoMode = rbMonoLeft->isChecked()?1:(rbMonoRight->isChecked()?2:3);
   _downsample = rbDS1->isChecked()?1:(rbDS2->isChecked()?2:3);
   _checkRangeViolations = cbRangeViolation->isChecked();
   _Resync = cbResync->isChecked();
   _changeScalefactor = cbScalefactor->isChecked();
   _scalefactor = QString(leScalefactor->text()).toInt();
   
   _useDefaultAudioDevice = rbDeviceDefault->isChecked();
   _audioDevice = leDevice->text();
   _outputTo = rbSpeaker->isChecked()?1:(rbLineout->isChecked()?2:3);
   _bufferSize = slider->value()*16;

   config->setGroup("General");
   config->writeEntry( "HTTPServer", _httpProxy );
   config->writeEntry( "HTTPPort", _httpPort );
   config->writeEntry( "PlayOnOpen", _playOnOpen );
   config->writeEntry( "ReadID3", _readID3 );
   config->writeEntry( "Autoscroll", _autoscroll );
   config->setGroup("Decoder");
   config->writeEntry( "ForceToMono", _forceToMono );
   config->writeEntry( "MonoMode", _monoMode );
   config->writeEntry( "Downsample", _downsample );
   config->writeEntry( "CheckRangeViolations", _checkRangeViolations );
   config->writeEntry( "Resync", _Resync );
   config->writeEntry( "ChangeScalefactor", _changeScalefactor );
   config->writeEntry( "Scalefactor", _scalefactor );
   config->setGroup("AudioDevice");
   config->writeEntry( "AudioDeviceDefault", _useDefaultAudioDevice );
   config->writeEntry( "AudioDevice", _audioDevice );
   config->writeEntry( "OutputTo", _outputTo );
   config->writeEntry( "Buffer", _bufferSize );
   config->sync();   

   emit( sigChanged() );
   prefsChanged = false;
}

void Preferences::slotShow()
{
   initPrefs();
   show();
}

void Preferences::slotOk()
{
   applyPrefs();
   hide();
}

void Preferences::slotApply()
{
   applyPrefs();
}

void Preferences::slotCancel()
{
   hide();
}

void Preferences::slotChange()
{
   prefsChanged = true;
   rbMonoLeft->setEnabled( rbMono->isChecked() );
   rbMonoRight->setEnabled( rbMono->isChecked() );
   rbMonoMix->setEnabled( rbMono->isChecked() );
   leScalefactor->setEnabled( cbScalefactor->isChecked() );
   leDevice->setEnabled( rbDeviceOther->isChecked() );
}

void Preferences::slotChange( int )
{
   slotChange();
}

void Preferences::slotChange( const char* )
{
   slotChange();
}

